# NEAT-JavaScript

This project is a JavaScript-based implementation of NEAT (Neuroevolution of Augmenting Topologies), an evolutionary algorithm developed by Kenneth O. Stanley and Risto Miikkulainen. Originally introduced in their 2002 paper, [Evolving Neural Networks Through Augmenting Topologies](https://nn.cs.utexas.edu/downloads/papers/stanley.ec02.pdf), NEAT presents a novel approach to evolving artificial neural networks by optimizing both network weights and structures over generations.

In this implementation, NEAT’s principles are faithfully applied, emphasizing the algorithm's core components: speciation, crossover, and structural mutation. These elements enable neural networks to adapt in complexity as they evolve, making NEAT a unique approach to neuroevolutionary algorithms.

This implementation offers a clear, accessible codebase for those exploring NEAT and has achieved results comparable to the benchmarks demonstrated in the original paper, providing a solid foundation for further exploration and development.

## Examples
- [Self Driving Car](https://neat-javascript.org/examples/self-driving-car.html)
- [Flappy Bird](https://neat-javascript.org/examples/flappy-bird.html)
- [Pole-Balancing](https://neat-javascript.org/examples/pole-balancing.html)
- [Ant Simulator](https://neat-javascript.org/examples/ant-simulator.html)

## Installation

### Node.js installation

To install for Node.js, run the following command:

```bash
npm install neat-javascript
```

### Browser Installation

To use in a browser, simply add the following script tag to your HTML file:

```html
<script src="https://neat-javascript.org/releases/NEAT-JavaScript-1.0.0.js"></script>
```

## Documentation
For a comprehensive guide on usage and complete documentation, visit [this](https://neat-javascript.org) link.

For a quick-start guide, check out [this](https://neat-javascript.org/quickstart.html) link.

## Contributing

We welcome contributions! Please open an issue first to discuss your proposed updates.

## License

This project is licensed under the [GPL-3.0](https://www.gnu.org/licenses/gpl-3.0.txt).